==> edge-login-ui-rn-0.9.23.tgz: package/lib/components/scenes/OtpErrorScene.js <==
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as React from 'react';
import FontAwesome from 'react-native-vector-icons/FontAwesome';
import { sprintf } from 'sprintf-js';
import { login } from '../../actions/LoginAction';
import { hasReadyVoucher, requestOtpReset } from '../../actions/LoginOtpActions';
import s from '../../common/locales/strings';
import { makePeriodicTask } from '../../util/periodicTask';
import { toLocalTime } from '../../util/utils';
import { showResetModal } from '../modals/OtpResetModal';
import { showQrCodeModal } from '../modals/QrCodeModal';
import { TextInputModal } from '../modals/TextInputModal';
import { Airship, showError, showToast } from '../services/AirshipInstance';
import { connect } from '../services/ReduxStore';
import { DividerWithText } from '../themed/DividerWithText';
import { IconHeaderRow } from '../themed/IconHeaderRow';
import { LinkRow } from '../themed/LinkRow';
import { ThemedScene } from '../themed/ThemedScene';
import { MessageText, Warning } from '../themed/ThemedText';
class OtpErrorSceneComponent extends React.Component {
    constructor() {
        super(...arguments);
        this.checkVoucher = makePeriodicTask(() => __awaiter(this, void 0, void 0, function* () {
            const { hasReadyVoucher, login, otpAttempt, otpError, saveOtpError } = this.props;
            try {
                const result = yield hasReadyVoucher(otpError);
                if (result) {
                    showToast(s.strings.otp_scene_retrying);
                    yield login(otpAttempt);
                }
            }
            catch (error) {
                if (error != null && error.name === 'OtpError') {
                    saveOtpError(otpAttempt, error);
                }
                else {
                    showError(error);
                }
            }
        }), 5000);
        this.handleBackupModal = () => {
            const { login, otpAttempt, saveOtpError } = this.props;
            const handleSubmit = (otpKey) => __awaiter(this, void 0, void 0, function* () {
                try {
                    this.checkVoucher.stop();
                    yield login(otpAttempt, otpKey);
                    return true;
                }
                catch (error) {
                    // Restart the background checking if the login failed:
                    this.checkVoucher.start();
                    // Translate known errors:
                    if (error != null && error.name === 'OtpError') {
                        saveOtpError(otpAttempt, error);
                        return s.strings.backup_key_incorrect;
                    }
                    if (error != null && error.message === 'Unexpected end of data') {
                        return s.strings.backup_key_incorrect;
                    }
                    showError(error);
                    return false;
                }
            });
            Airship.show(bridge => (React.createElement(TextInputModal, { bridge: bridge, onSubmit: handleSubmit, title: s.strings.otp_backup_code_modal_title, message: s.strings.otp_instructions, inputLabel: s.strings.backup_key_label, submitLabel: s.strings.submit, autoCapitalize: "characters", returnKeyType: "done" })));
        };
    }
    componentDidMount() {
        this.checkVoucher.start();
    }
    componentWillUnmount() {
        this.checkVoucher.stop();
    }
    render() {
        const { handleQrModal, otpError, otpResetDate } = this.props;
        const isIp = otpError.reason === 'ip';
        // Find the automatic login date:
        let date = otpResetDate;
        if (otpError.voucherActivates != null)
            date = otpError.voucherActivates;
        return (React.createElement(ThemedScene, { onBack: this.props.onBack, title: isIp ? s.strings.otp_header_ip : s.strings.otp_header },
            React.createElement(IconHeaderRow, { renderIcon: theme => (React.createElement(Warning, null,
                    React.createElement(FontAwesome, { name: "exclamation-triangle", size: theme.rem(2.5) }))) },
                React.createElement(MessageText, null,
                    React.createElement(Warning, null, isIp
                        ? s.strings.otp_scene_header_ip
                        : s.strings.otp_scene_header_2fa))),
            React.createElement(DividerWithText, { label: s.strings.to_fix }),
            React.createElement(MessageText, null, s.strings.otp_scene_approve),
            React.createElement(DividerWithText, null),
            React.createElement(LinkRow, { label: s.strings.otp_scene_qr, onPress: handleQrModal }),
            isIp ? null : (React.createElement(React.Fragment, null,
                React.createElement(DividerWithText, null),
                React.createElement(LinkRow, { label: s.strings.otp_backup_code_modal_title, onPress: this.handleBackupModal }))),
            date == null ? null : (React.createElement(React.Fragment, null,
                React.createElement(DividerWithText, null),
                React.createElement(MessageText, null, sprintf(s.strings.otp_scene_wait, toLocalTime(date))))),
            otpError.resetToken == null || date != null ? null : (React.createElement(React.Fragment, null,
                React.createElement(DividerWithText, null),
                React.createElement(LinkRow, { label: s.strings.disable_otp_button_two, onPress: () => showResetModal(this.props.requestOtpReset) })))));
    }
}
export const OtpErrorScene = connect((state) => {
    const { otpAttempt, otpError, otpResetDate } = state.login;
    if (otpAttempt == null || otpError == null) {
        throw new Error('Missing OtpError for OTP error scene');
    }
    return { otpAttempt, otpError, otpResetDate };
}, (dispatch) => ({
    onBack() {
        dispatch({ type: 'START_PASSWORD_LOGIN' });
    },
    handleQrModal() {
        dispatch(showQrCodeModal());
    },
    hasReadyVoucher(error) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield dispatch(hasReadyVoucher(error));
        });
    },
    login(attempt, otpKey) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield dispatch(login(attempt, otpKey));
        });
    },
    requestOtpReset() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield dispatch(requestOtpReset());
        });
    },
    saveOtpError(attempt, error) {
        dispatch({ type: 'OTP_ERROR', data: { attempt, error } });
    }
}))(OtpErrorSceneComponent);

==> edge-login-ui-rn-2.1.0.tgz: package/lib/components/scenes/OtpErrorScene.js <==
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { asMaybeOtpError } from 'edge-core-js';
import * as React from 'react';
import FontAwesome from 'react-native-vector-icons/FontAwesome';
import { sprintf } from 'sprintf-js';
import { completeLogin } from '../../actions/LoginCompleteActions';
import s from '../../common/locales/strings';
import { useImports } from '../../hooks/useImports';
import { useDispatch } from '../../types/ReduxTypes';
import { attemptLogin } from '../../util/loginAttempt';
import { makePeriodicTask } from '../../util/periodicTask';
import { toLocalTime } from '../../util/utils';
import { showResetModal } from '../modals/OtpResetModal';
import { showQrCodeModal } from '../modals/QrCodeModal';
import { TextInputModal } from '../modals/TextInputModal';
import { Airship, showError, showToast } from '../services/AirshipInstance';
import { DividerWithText } from '../themed/DividerWithText';
import { IconHeaderRow } from '../themed/IconHeaderRow';
import { LinkRow } from '../themed/LinkRow';
import { ThemedScene } from '../themed/ThemedScene';
import { MessageText, Warning } from '../themed/ThemedText';
class OtpErrorSceneComponent extends React.Component {
    constructor() {
        super(...arguments);
        this.checkVoucher = makePeriodicTask(() => __awaiter(this, void 0, void 0, function* () {
            const { hasReadyVoucher, login, otpAttempt, otpError, saveOtpError } = this.props;
            try {
                const result = yield hasReadyVoucher(otpError);
                if (result) {
                    showToast(s.strings.otp_scene_retrying);
                    yield login(otpAttempt);
                }
            }
            catch (error) {
                const otpError = asMaybeOtpError(error);
                if (otpError != null) {
                    saveOtpError(otpAttempt, otpError);
                }
                else {
                    showError(error);
                }
            }
        }), 5000);
        this.handleBackupModal = () => {
            const { login, otpAttempt, saveOtpError } = this.props;
            const handleSubmit = (otpKey) => __awaiter(this, void 0, void 0, function* () {
                try {
                    this.checkVoucher.stop();
                    yield login(otpAttempt, otpKey);
                    return true;
                }
                catch (error) {
                    // Restart the background checking if the login failed:
                    this.checkVoucher.start();
                    // Translate known errors:
                    const otpError = asMaybeOtpError(error);
                    if (otpError != null) {
                        saveOtpError(otpAttempt, otpError);
                        return s.strings.backup_key_incorrect;
                    }
                    if (error instanceof Error &&
                        error.message === 'Unexpected end of data') {
                        return s.strings.backup_key_incorrect;
                    }
                    showError(error);
                    return false;
                }
            });
            Airship.show(bridge => (React.createElement(TextInputModal, { bridge: bridge, onSubmit: handleSubmit, title: s.strings.otp_backup_code_modal_title, message: s.strings.otp_instructions, inputLabel: s.strings.backup_key_label, submitLabel: s.strings.submit, autoCapitalize: "characters", returnKeyType: "done" })));
        };
    }
    componentDidMount() {
        this.checkVoucher.start();
    }
    componentWillUnmount() {
        this.checkVoucher.stop();
    }
    render() {
        const { handleQrModal, otpError, otpResetDate } = this.props;
        const isIp = otpError.reason === 'ip';
        // Find the automatic login date:
        let date = otpResetDate;
        if (otpError.voucherActivates != null)
            date = otpError.voucherActivates;
        return (React.createElement(ThemedScene, { onBack: this.props.onBack, title: isIp ? s.strings.otp_header_ip : s.strings.otp_header },
            React.createElement(IconHeaderRow, { renderIcon: theme => (React.createElement(Warning, null,
                    React.createElement(FontAwesome, { name: "exclamation-triangle", size: theme.rem(2.5) }))) },
                React.createElement(MessageText, null,
                    React.createElement(Warning, null, isIp
                        ? s.strings.otp_scene_header_ip
                        : s.strings.otp_scene_header_2fa))),
            React.createElement(DividerWithText, { label: s.strings.to_fix }),
            React.createElement(MessageText, null, s.strings.otp_scene_approve),
            React.createElement(DividerWithText, null),
            React.createElement(LinkRow, { label: s.strings.otp_scene_qr, onPress: handleQrModal }),
            isIp ? null : (React.createElement(React.Fragment, null,
                React.createElement(DividerWithText, null),
                React.createElement(LinkRow, { label: s.strings.otp_backup_code_modal_title, onPress: this.handleBackupModal }))),
            date == null ? null : (React.createElement(React.Fragment, null,
                React.createElement(DividerWithText, null),
                React.createElement(MessageText, null, sprintf(s.strings.otp_scene_wait, toLocalTime(date))))),
            otpError.resetToken == null || date != null ? null : (React.createElement(React.Fragment, null,
                React.createElement(DividerWithText, null),
                React.createElement(LinkRow, { label: s.strings.disable_otp_button_two, onPress: () => showResetModal(this.props.requestOtpReset) })))));
    }
}
export function OtpErrorScene(props) {
    const { route } = props;
    const { otpAttempt, otpError } = route.params;
    const { accountOptions, context } = useImports();
    const dispatch = useDispatch();
    const [otpResetDate, setOtpResetDate] = React.useState(otpError.resetDate);
    const handleBack = () => {
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'passwordLogin', params: {} }
        });
    };
    const handleQrModal = () => {
        dispatch(showQrCodeModal());
    };
    function hasReadyVoucher() {
        return __awaiter(this, void 0, void 0, function* () {
            const { voucherId } = otpError;
            if (voucherId == null)
                return false;
            // Is that voucher pending?
            const messages = yield context.fetchLoginMessages();
            for (const message of messages) {
                const { pendingVouchers } = message;
                for (const voucher of pendingVouchers) {
                    if (voucher.voucherId === voucherId)
                        return false;
                }
            }
            return true;
        });
    }
    function requestOtpReset() {
        return __awaiter(this, void 0, void 0, function* () {
            const { resetToken } = otpError;
            if (resetToken == null) {
                throw new Error('No OTP reset token');
            }
            const date = yield context.requestOtpReset(otpAttempt.username, resetToken);
            setOtpResetDate(date);
        });
    }
    function saveOtpError(otpAttempt, otpError) {
        setOtpResetDate(otpError.resetDate);
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'otpError', params: { otpAttempt, otpError } }
        });
    }
    function login(attempt, otpKey) {
        return __awaiter(this, void 0, void 0, function* () {
            const account = yield attemptLogin(context, attempt, Object.assign(Object.assign({}, accountOptions), { otpKey }));
            dispatch(completeLogin(account));
        });
    }
    return (React.createElement(OtpErrorSceneComponent, { handleQrModal: handleQrModal, hasReadyVoucher: hasReadyVoucher, login: login, otpAttempt: otpAttempt, otpError: otpError, otpResetDate: otpResetDate, requestOtpReset: requestOtpReset, route: route, saveOtpError: saveOtpError, onBack: handleBack }));
}
